/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup netif
 *  @{
 */

/**
 * @file
 */

#ifndef __NET_NETIF_MESSAGES_H__
#define __NET_NETIF_MESSAGES_H__

#include <async.h>
#include <errno.h>
#include <malloc.h>

#include <ipc/ipc.h>

#include "../messages.h"

#include "../structures/measured_strings.h"
#include "../structures/packet/packet.h"

#include "device.h"

typedef enum {
	/* ( device_id, irq, io ) */
	NET_NETIF_PROBE = NET_NETIF_FIRST,
	/* () not supported, should ask networking for a name and register device */
	NET_NETIF_PROBE_AUTO,
	/* ( device_id, packet_id ) */
	NET_NETIF_SEND,
	/* ( device_id ) */
	NET_NETIF_START,
	/* ( device_id ), ipc_data_read( stats ) */
	NET_NETIF_STATS,
	/* ( device_id ) */
	NET_NETIF_STOP,
	/* */
	NET_NETIF_SET_ADDR,
	/* */
	NET_NETIF_GET_ADDR,
} netif_messages;

#ifdef NETIF_BUNDLE

#include "../netif/netif_wrappers.h"

#define NETIF_GET_IRQ( call )		( int )			IPC_GET_ARG2( * call )

#define NETIF_GET_IO( call )		( int )			IPC_GET_ARG3( * call )

#define netif_get_addr( netif_phone, device_id, address, data )	\
	netif_get_addr_wrapper( device_id, address, data )

static inline int	netif_probe_req( int netif_phone, device_id_t device_id, int irq, int io ){
	return netif_probe_wrapper( device_id, irq, io );
}

#define netif_send_msg( netif_phone, device_id, packet, sender )	\
	netif_send_wrapper( device_id, packet, sender )

static inline int	netif_start_req( int netif_phone, device_id_t device_id ){
	return netif_start_wrapper( device_id );
}

static inline int	netif_stop_req( int netif_phone, device_id_t device_id ){
	return netif_stop_wrapper( device_id );
}

#else

#define netif_get_addr( netif_phone, device_id, address, data )	\
	generic_get_addr( netif_phone, NET_NETIF_GET_ADDR, device_id, address, data )

static inline int	netif_probe_req( int netif_phone, device_id_t device_id, int irq, int io ){
	return async_req_3_0( netif_phone, NET_NETIF_PROBE, device_id, irq, io );
}

#define netif_send_msg( netif_phone, device_id, packet, sender )	\
	generic_send_msg( netif_phone, NET_NETIF_SEND, device_id, packet_get_id( packet ), sender )

static inline int	netif_start_req( int netif_phone, device_id_t device_id ){
	return async_req_1_0( netif_phone, NET_NETIF_START, device_id );
}

static inline int	netif_stop_req( int netif_phone, device_id_t device_id ){
	return async_req_1_0( netif_phone, NET_NETIF_STOP, device_id );
}

#endif

#endif

/** @}
 */
